<?php
class Billing extends Backend_Controller {
	
	public function __construct() 
	{
		parent::__construct();
		$this->data['_menu'] = 'billing';
		$this->load->model('project_mdl');
		$this->load->model('invoice_mdl');
		$this->load->model('invoice_items_mdl');
	}
	
	public function index() 
	{
		//Get invoices from the database
		$this->db->join('projects', 'projects.project_id = invoices.project_id');
		$this->db->join('clients', 'clients.client_id = invoices.client_id');
		
		//Get all invoices from the database
		$count = $this->db->count_all_results('invoices');
		
		// Set pagination
		$perpage = $this->pagination_config['per_page'];
		if ($count > $perpage) {
			$this->pagination_config['base_url'] = site_url($this->uri->segment(1) .'/index/');
			$this->pagination_config['total_rows'] = $count;
			$this->pagination_config['uri_segment'] = 3;
			$this->pagination->initialize($this->pagination_config);
			$this->data['pagination'] = $this->pagination->create_links();
			$offset = $this->uri->segment(3);
		}
		else{
			$this->data['pagination'] = '';
			$offset = 0;
		}
		//Get invoices from the database
		$this->db->limit($perpage, $offset);
		$this->db->join('projects', 'projects.project_id = invoices.project_id');
		$this->db->join('clients', 'clients.client_id = invoices.client_id');
		
		$invoices = $this->invoice_mdl->get();
		$invoice_records = array ();
		$this->load->model('invoice_items_mdl');
			foreach ($invoices as $count => $invoice) :
				$invoice_records[$count]['client_name']		=	$invoice->firstname . ' ' . $invoice->lastname;
				$invoice_records[$count]['project_name']	=	$invoice->project_name;
				$invoice_records[$count]['invoice_number']	=	$invoice->invoice_number;
				$invoice_records[$count]['invoice_status']	=	$invoice->invoice_status;
				$invoice_records[$count]['invoice_amount']	=	format_amount($this->invoice_mdl->get_invoice_total_amt($invoice->invoice_id), true);
				$invoice_records[$count]['paid_amount']		=	format_amount($this->invoice_mdl->get_invoice_paid_amt($invoice->invoice_id), true);
				
				$invoice_records[$count]['invoice_id']		=	$invoice->invoice_id;
			endforeach;

		$this->db->where('project_status', 'open');
		$this->data['projects'] = $this->project_mdl->projects_dropdown();
		$this->data['invoices'] = $invoice_records;
		$this->data['pagecontent'] = 'billing/invoices';
		
		if(IS_AJAX)
			$this->load->view('billing/invoices', $this->data);
		else 
			$this->load->view('components/_layout', $this->data);
		
	}
	
	public function invoice_template(){
		$this->load->model('timer_mdl');
		$this->load->model('settings_mdl');
		$project_id = $this->input->post('project'); 
		
		// Get the project details
		$this->db->select('*, clients.hourly_rate AS client_hourly_rate, projects.hourly_rate AS project_hourly_rate');
		$this->db->join('clients', 'clients.client_id = projects.client_id', 'LEFT');
		$this->db->where('project_id', $project_id);
		$project_details = $this->project_mdl->get(null, true);
		
		//set the client information
		$client = new stdClass();
		$client->name 		= ucwords($project_details->firstname . ' ' . $project_details->lastname);
		$client->address 	= $project_details->address;
		$client->city 		= $project_details->city;
		$client->country 	= country_name($project_details->country);
		$client->company 	= $project_details->company;
		$client->client_id 	= $project_details->client_id;
		$this->data['client'] = $client;
		
		//Set project information
		$project = new stdClass();
		$project->project_name = $project_details->project_name;
		$project->project_id   = $project_details->project_id;
		$this->data['project'] = $project;
		
		//Get the time entries for that project
		$this->db->where('is_invoiced', '0');
		$this->db->where('stop_time != ', 0);
		$this->db->where('project_id', $project_id);
		$time_entries = $this->timer_mdl->get();
		
		$invoice_items = array();
		$invoice_tax 		= 0;
		$invoice_subtotal 	= 0;
		foreach ($time_entries as $count => $entry) : 
			$invoice_items[$count]['entry_id']     = $entry->id;
			$invoice_items[$count]['description']  = ($entry->task != '') ? $entry->task .' - Starting '. date("M j, Y", $entry->start_time) . ' at ' . date("g:i A", $entry->start_time) : 'Work Starting '. date("M j, Y", $entry->start_time) . ' at ' . date("g:i A", $entry->start_time);
			$invoice_items[$count]['hours_worked'] =  time_elapsed($entry->start_time, $entry->stop_time);
			$invoice_items[$count]['hourly_rate']  =  $project_details->project_hourly_rate;
			$invoice_items[$count]['item_amount']  =  $invoice_items[$count]['hours_worked'] * $invoice_items[$count]['hourly_rate'];
			$invoice_subtotal 					  +=  $invoice_items[$count]['item_amount'];
		endforeach;
		
		$this->data['invoice_items'] = $invoice_items;
		
		$invoice_total = $invoice_subtotal + $invoice_tax;
		
		$this->data['invoice_items'] = $invoice_items;
		$this->data['invoice_tax']   = format_amount ($invoice_tax);
		$this->data['invoice_subtotal']   = format_amount ($invoice_subtotal);
		$this->data['invoice_total'] = format_amount ($invoice_total);
		
		//get system settings information
		$this->data['settings']  = $this->settings_mdl->get(null, true);
		
		$template =  $this->load->view('billing/invoice_template', $this->data, true);
		$response = array('success' => true, 'template' => $template);
		echo json_encode($response);
	}
	
	public function previewinvoice()
	{
		$this->load->model('invoice_mdl');
		$this->load->model('timer_mdl');
		$this->load->model('settings_mdl');
		$project_id = $this->input->post('project');
		$invoice_id = $this->input->post('invoice_id');
		
		$items = json_decode($this->input->post('invoice_items'));
		
		// Get the project details
		$this->db->select('*, clients.hourly_rate AS client_hourly_rate, projects.hourly_rate AS project_hourly_rate');
		$this->db->join('clients', 'clients.client_id = projects.client_id', 'LEFT');
		$this->db->where('project_id', $project_id);
		$project_details = $this->project_mdl->get(null, true);
		
		//set the client information
		$client = new stdClass();
		$client->name 		= ucwords($project_details->firstname . ' ' . $project_details->lastname);
		$client->address 	= $project_details->address;
		$client->city 		= $project_details->city;
		$client->country 	= country_name($project_details->country);
		$client->company 	= $project_details->company;
		$this->data['client'] = $client;
		
		//Set project information
		$project = new stdClass();
		$project->project_name = $project_details->project_name;
		$this->data['project'] = $project;

		$invoice_items = array();
		$invoice_tax 	= 0; 
		$invoice_subtotal 	= 0;
		
		foreach ($items as $count => $entry) :
		$invoice_items[$count]['description']  = $entry->item_description;
		$invoice_items[$count]['hours_worked'] = $entry->item_hours_worked;
		$invoice_items[$count]['hourly_rate']  = $entry->hourly_rate;
		$invoice_items[$count]['item_amount']  = $entry->item_subtotal;
		$invoice_tax     						+= ($entry->item_tax * $entry->item_subtotal ) / 100 ;
		$invoice_subtotal 						+= $entry->item_subtotal;
		endforeach;
		
		$invoice_total = $invoice_subtotal + $invoice_tax;
		
		$this->data['invoice_items'] = $invoice_items;
		$this->data['invoice_tax']   = format_amount ($invoice_tax, true);
		$this->data['invoice_subtotal']   = format_amount ($invoice_subtotal, true);
		$this->data['invoice_total'] = format_amount ($invoice_total, true);
		$this->data['due_date'] = $this->invoice_due_date();
		
		//get system settings information
		$this->data['settings']  = $this->settings_mdl->get(null, true);
		
		//set the existing invoice details
		$invoice_details = $this->invoice_mdl->get($invoice_id, true);
		$this->data['invoice_num'] = (!empty($invoice_details)) ? $invoice_details->invoice_number : '0000';
		$this->data['invoice_date'] = (!empty($invoice_details)) ? $invoice_details->created : date('Y-m-d');
		$this->data['invoice_due_date'] = (!empty($invoice_details)) ? $invoice_details->due_date : $this->invoice_due_date();
		
		$invoice_preview =  $this->load->view('pdf_templates/invoice_template', $this->data, true);
		$response = array('preview' => $invoice_preview );
		echo json_encode($response);
	}
	
	public function create_invoice(){
		$save_type 	= $this->input->post('save_type');
		$items 		= json_decode($this->input->post('invoice_items'));
		$client 	= $this->input->post('client_id');
		$project 	= $this->input->post('project_id');
		$status 	= ($save_type == 'draft') ? 'DRAFT' : 'UNPAID';
		
		$invoice_details = array(
			'invoice_number' 	=> '0000', 
			'client_id' 		=> $client,
			'project_id' 		=> $project,
			'invoice_status' 	=> $status,
			'due_date' 			=> $this->invoice_due_date(),
		);
		$invoice_id = $this->invoice_mdl->save($invoice_details);
		$invoice_num = array('invoice_number' => str_pad($invoice_id, 4, "0", STR_PAD_LEFT));
		$invoice_id = $this->invoice_mdl->save($invoice_num, $invoice_id);
		
		$this->load->model('invoice_items_mdl');
		foreach ($items as $count => $entry) :
		$amount 	= $entry->item_hours_worked * $entry->hourly_rate;
		$invoice_item = array(
			'invoice_id'		=> $invoice_id,
			'time_entry_id'		=> $entry->entry_id,
			'description'		=> $entry->item_description,
			'hours'				=> $entry->item_hours_worked,
			'rate'				=> $entry->hourly_rate,
			'tax'				=> $entry->item_tax,
			'amount'			=> $amount,
		);
		$item_id = $this->invoice_items_mdl->save($invoice_item);
		endforeach;
		
		if($save_type == 'new'){
			//code for sending pdf invoice via email to client
			$attachment = $this->invoice_pdf($invoice_id, false);
			$this->notifications->invoice_generated($invoice_id, $attachment);
			//update all fields in the items table to is_invoiced = 1
			$this->load->model('timer_mdl');
			foreach ($items as $count => $entry) :
				$time_entry = array('is_invoiced' => '1');
				$this->timer_mdl->save($time_entry, $entry->entry_id);
			endforeach;
			
		}
		
		if($item_id) :
			$this->session->set_flashdata('success', 'Invoice saved successfully');
			$response = array ('success' => true);
		else :
			$this->session->set_flashdata('error', 'Error occured while saving the invoice');
			$response = array ('success' => false);
		endif;
		
		echo json_encode($response);
	}
	
	public function edit($invoice_id = null){
		$this->load->model('settings_mdl');
		$this->load->model('invoice_mdl');
		$this->load->model('invoice_items_mdl');
		
		if(!empty($_POST)) :
			$invoice_items 		= json_decode($this->input->post('invoice_items'));
			$this->load->model('invoice_items_mdl');
			foreach ($invoice_items as $count => $entry) :
			$amount 	= $entry->item_hours_worked * $entry->hourly_rate;
			$invoice_item = array(
					'description'		=> $entry->item_description,
					'hours'				=> $entry->item_hours_worked,
					'rate'				=> $entry->hourly_rate,
					'tax'				=> $entry->item_tax,
					'amount'			=> $amount,
			);
			if($entry->item_id != '')
			{
				$item_id = $this->invoice_items_mdl->save($invoice_item, $entry->item_id);
				$response = array('success' => true);
				$this->session->set_flashdata('success', 'Invoice updated successfully');
			}
			else{
				$invoice_item['invoice_id'] = $invoice_id;
				$invoice_item['time_entry_id'] = $entry->entry_id;
				$item_id = $this->invoice_items_mdl->save($invoice_item);
				$response = array('success' => true);
				$this->session->set_flashdata('success', 'Invoice updated successfully');
			}
			endforeach;
		endif;
		
		// Get the project details
		$this->db->select('*, clients.hourly_rate AS client_hourly_rate, projects.hourly_rate AS project_hourly_rate');
		$this->db->join('clients', 'clients.client_id = invoices.client_id', 'LEFT');
		$this->db->join('projects', 'projects.project_id = invoices.project_id', 'LEFT');
		$this->db->where('invoice_id', $invoice_id);
		$invoice_details = $this->invoice_mdl->get(null, true);
		
		//set the client information
		$client = new stdClass();
		$client->name 		= ucwords($invoice_details->firstname . ' ' . $invoice_details->lastname);
		$client->address 	= $invoice_details->address;
		$client->city 		= $invoice_details->city;
		
		$client->country 	= country_name($invoice_details->country);
		$client->company 	= $invoice_details->company;
		$client->client_id 	= $invoice_details->client_id;
		$client->invoice_id = $invoice_details->invoice_id;
		$this->data['client'] = $client;
		
		//Set project information
		$project = new stdClass();
		$project->project_name = $invoice_details->project_name;
		$project->project_id   = $invoice_details->project_id;
		$this->data['project'] = $project;
		
		//Get the time entries for that project
		$this->db->where('invoice_id = ', $invoice_id);
		$time_entries = $this->invoice_items_mdl->get();
		
		$invoice_items = array();
		$invoice_tax 		= 0;
		$invoice_subtotal 	= 0;
		foreach ($time_entries as $count => $entry) :
		$invoice_items[$count]['entry_id']	   = $entry->time_entry_id;
		$invoice_items[$count]['item_id']      = $entry->item_id;
		$invoice_items[$count]['description']  = $entry->description;
		$invoice_items[$count]['hours_worked'] = $entry->hours;
		$invoice_items[$count]['hourly_rate']  = $entry->rate;
		$invoice_items[$count]['item_amount']  = $entry->amount;
		$invoice_items[$count]['item_tax'] 	   = $entry->tax;
		$invoice_subtotal 					  += $invoice_items[$count]['item_amount'];
		$invoice_tax 						  += ($entry->tax * $invoice_items[$count]['item_amount']) / 100;
		endforeach;
		
		$this->data['invoice_items'] = $invoice_items;
		
		$invoice_total = $invoice_subtotal + $invoice_tax;
		
		$this->data['invoice_items'] = $invoice_items;
		$this->data['invoice_tax']   = format_amount ($invoice_tax);
		$this->data['invoice_subtotal']   = format_amount ($invoice_subtotal);
		$this->data['invoice_total'] = format_amount ($invoice_total);
		$this->data['invoice_number'] = $invoice_details->invoice_number;
		
		//get system settings information
		$this->data['settings']  = $this->settings_mdl->get(null, true);
		
		$template =  $this->load->view('billing/invoice_template', $this->data, true);
		$this->data['pagecontent'] = 'billing/edit_invoice';
		
		if(IS_AJAX)
			echo json_encode($response);
		else		
			$this->load->view('components/_layout', $this->data);
	}
	
	public function delete_invoice_item($invoice_id = null, $invoice_item_id = null){
		$this->load->model('invoice_items_mdl');
		$this->invoice_items_mdl->delete($invoice_item_id);
		$this->session->set_flashdata('success', 'The item has been deleted successfully !!');
		redirect('billing/edit/'.$invoice_id);
	}
/*---------------------------------------------------------------------------------------------------------
 | Function to set the invoice due date
|----------------------------------------------------------------------------------------------------------*/

	public function invoice_due_date(){
		$this->load->model('settings_mdl');
		$settings  = $this->settings_mdl->get(null, true);
		$invoice_due_num_days = $settings->invoice_due_days;
		$now = date('Y-m-d H:i:s');
		$due_date = date('Y-m-d H:i:s',strtotime($now . "+" . $invoice_due_num_days ." days"));
		return $due_date;
	}
/*---------------------------------------------------------------------------------------------------------
 | Function to delete invoices
|----------------------------------------------------------------------------------------------------------*/
	public function delete($invoice_id = null){
		$this->invoice_mdl->delete_invoice($invoice_id);
		$this->session->set_flashdata('success', 'The invoice has been deleted !!');
		redirect('billing');
	}

/*---------------------------------------------------------------------------------------------------------
 | Function to display the pdf invoice
|----------------------------------------------------------------------------------------------------------*/
	public function invoice_pdf($invoice_id = null, $stream = true){
		$this->load->library("pdf");
		$this->load->model('settings_mdl');
		// Get the project details
		$this->db->select('*, clients.hourly_rate AS client_hourly_rate, projects.hourly_rate AS project_hourly_rate');
		$this->db->join('clients', 'clients.client_id = invoices.client_id', 'LEFT');
		$this->db->join('projects', 'projects.project_id = invoices.project_id', 'LEFT');
		$this->db->where('invoice_id', $invoice_id);
		$invoice_details = $this->invoice_mdl->get(null, true);
		
		//set the client information
		$client = new stdClass();
		$client->name 		= ucwords($invoice_details->firstname . ' ' . $invoice_details->lastname);
		$client->address 	= $invoice_details->address;
		$client->city 		= $invoice_details->city;
		
		$client->country 	= country_name($invoice_details->country);
		$client->company 	= $invoice_details->company;
		$client->client_id 	= $invoice_details->client_id;
		$client->invoice_id = $invoice_details->invoice_id;
		$this->data['client'] = $client;
		
		//Set project information
		$project = new stdClass();
		$project->project_name = $invoice_details->project_name;
		$project->project_id   = $invoice_details->project_id;
		$this->data['project'] = $project;
		
		//Get the time entries for that project
		$this->db->where('invoice_id = ', $invoice_id);
		$time_entries = $this->invoice_items_mdl->get();
		$invoice_items = array();
		$invoice_tax 		= 0;
		$invoice_subtotal 	= 0;
		foreach ($time_entries as $count => $entry) :
		$invoice_items[$count]['entry_id']	   = $entry->time_entry_id;
		$invoice_items[$count]['item_id']      = $entry->item_id;
		$invoice_items[$count]['description']  = $entry->description;
		$invoice_items[$count]['hours_worked'] = $entry->hours;
		$invoice_items[$count]['hourly_rate']  = $entry->rate;
		$invoice_items[$count]['item_amount']  = $entry->amount;
		$invoice_items[$count]['item_tax'] 	   = $entry->tax;
		$invoice_subtotal 					  += $invoice_items[$count]['item_amount'];
		$invoice_tax 						  += ($entry->tax * $invoice_items[$count]['item_amount']) / 100;
		endforeach;
		
		$this->data['invoice_items'] 	= $invoice_items;
		$this->data['invoice_num'] 		= $invoice_details->invoice_number;
		$this->data['invoice_date'] 	= $invoice_details->created;
		$this->data['invoice_due_date'] = $invoice_details->due_date;
		$this->data['invoice_status'] 	= $invoice_details->invoice_status;
		
		$invoice_total = $this->invoice_mdl->get_invoice_total_amt($invoice_id);
		$invoice_paid_amt = $this->invoice_mdl->get_invoice_paid_amt($invoice_id);
		$invoice_amt_due = $invoice_total - $invoice_paid_amt;
		
		$this->data['invoice_items'] = $invoice_items;
		$this->data['invoice_tax']   = format_amount ($invoice_tax, true);
		$this->data['invoice_subtotal']   = format_amount ($invoice_subtotal, true);
		$this->data['invoice_total'] = format_amount ($invoice_total, true);
		$this->data['invoice_paid_amt'] = format_amount ($invoice_paid_amt, true);
		$this->data['invoice_amt_due'] = format_amount ($invoice_amt_due, true);
		
		//get system settings information
		$this->data['settings']  = $this->settings_mdl->get(null, true);
		$pdf_content =  $this->load->view('pdf_templates/invoice_pdf', $this->data, true);
		$filename = 'invoice_'. $invoice_details->invoice_number;
		$pdf_invoice = $this->pdf->create_pdf($filename, $pdf_content, $stream, false);
		return $pdf_invoice;
	}
/*---------------------------------------------------------------------------------------------------------
 | Functio to send invoice to client
|----------------------------------------------------------------------------------------------------------*/
	public function send_invoice($invoice_id = null)
	{
		$attachment = $this->invoice_pdf($invoice_id, false);
		$this->notifications->invoice_generated($invoice_id, $attachment);
		$this->invoice_mdl->set_invoice_status($invoice_id);
		$this->session->set_flashdata('success', 'The invoice has been sent to client !!');
		redirect('billing/edit/'.$invoice_id);
	}
/*---------------------------------------------------------------------------------------------------------
 | This is a sample comment to be used in commenting
|----------------------------------------------------------------------------------------------------------*/

}