<?php
class Cinvoices extends Frontend_Controller {
	protected  $client_id = 0;
	function __construct() 
	{
		parent::__construct();
		$this->data['_menu'] = 'invoices'; 
		$this->client_id = get_client_id();
		$this->load->model('invoice_mdl');
		$this->load->model('invoice_items_mdl');
	}
	
	public function index(){
		//Get all invoices from the database
		$this->db->where('invoice_status != ', 'DRAFT');
		$this->db->where('client_id', $this->client_id);
		$count = $this->db->count_all_results('invoices');
		
		// Set pagination
		$perpage = $this->pagination_config['per_page'];
		if ($count > $perpage) {
			$this->pagination_config['base_url'] = site_url($this->uri->segment(1) .'/index/');
			$this->pagination_config['total_rows'] = $count;
			$this->pagination_config['uri_segment'] = 3;
			$this->pagination->initialize($this->pagination_config);
			$this->data['pagination'] = $this->pagination->create_links();
			$offset = $this->uri->segment(3);
		}
		else{
			$this->data['pagination'] = '';
			$offset = 0;
		}
		//Get invoices from the database
		$this->db->limit($perpage, $offset);
		$this->db->join('projects', 'projects.project_id = invoices.project_id');
		$this->db->join('clients', 'clients.client_id = invoices.client_id');
		$this->db->where('invoices.client_id', $this->client_id);
		$this->db->where('invoice_status != ', 'DRAFT');
		$invoices = $this->invoice_mdl->get();
		
		$invoice_records = array ();
		$this->load->model('invoice_items_mdl');
			foreach ($invoices as $count => $invoice) :
				$invoice_records[$count]['client_name']		=	$invoice->firstname . ' ' . $invoice->lastname;
				$invoice_records[$count]['project_name']	=	$invoice->project_name;
				$invoice_records[$count]['invoice_number']	=	$invoice->invoice_number;
				$invoice_records[$count]['invoice_status']	=	$invoice->invoice_status;
				$invoice_records[$count]['invoice_amount']	=	format_amount($this->invoice_mdl->get_invoice_total_amt($invoice->invoice_id), true);
				$invoice_records[$count]['paid_amount']		=	format_amount($this->invoice_mdl->get_invoice_paid_amt($invoice->invoice_id), true);
				$invoice_records[$count]['invoice_id']		=	$invoice->invoice_id;
			endforeach;
		
		$this->data['invoices'] = $invoice_records;
		$this->data['pagecontent'] = 'frontend/invoices';
		
		if(IS_AJAX)
			$this->load->view('frontend/invoices', $this->data);
		else
			$this->load->view('components/_layout', $this->data);
	}
	
	public function view($invoice_id = null){
		$this->load->model('settings_mdl');	
		// Get the project details
		$this->db->select('*, clients.hourly_rate AS client_hourly_rate, projects.hourly_rate AS project_hourly_rate');
		$this->db->join('clients', 'clients.client_id = invoices.client_id', 'LEFT');
		$this->db->join('projects', 'projects.project_id = invoices.project_id', 'LEFT');
		$this->db->where('invoice_id', $invoice_id);
		$invoice_details = $this->invoice_mdl->get(null, true);
		
		//set the client information
		$client = new stdClass();
		$client->name 		= ucwords($invoice_details->firstname . ' ' . $invoice_details->lastname);
		$client->address 	= $invoice_details->address;
		$client->city 		= $invoice_details->city;
		$client->country 	= country_name($invoice_details->country);
		$client->company 	= $invoice_details->company;
		$client->client_id 	= $invoice_details->client_id;
		$client->invoice_id = $invoice_details->invoice_id;
		$this->data['client'] = $client;
		
		//Set project information
		$project = new stdClass();
		$project->project_name = $invoice_details->project_name;
		$project->project_id   = $invoice_details->project_id;
		$this->data['project'] = $project;
		
		//Get the time entries for that project
		$this->db->where('invoice_id = ', $invoice_id);
		$time_entries = $this->invoice_items_mdl->get();
		$invoice_items = array();
		$invoice_tax 		= 0;
		$invoice_subtotal 	= 0;
		foreach ($time_entries as $count => $entry) :
		$invoice_items[$count]['entry_id']	   = $entry->time_entry_id;
		$invoice_items[$count]['item_id']      = $entry->item_id;
		$invoice_items[$count]['description']  = $entry->description;
		$invoice_items[$count]['hours_worked'] = $entry->hours;
		$invoice_items[$count]['hourly_rate']  = $entry->rate;
		$invoice_items[$count]['item_amount']  = $entry->amount;
		$invoice_items[$count]['item_tax'] 	   = $entry->tax;
		$invoice_subtotal 					  += $invoice_items[$count]['item_amount'];
		$invoice_tax 						  += ($entry->tax * $invoice_items[$count]['item_amount']) / 100;
		endforeach;
		
		$this->data['invoice_items'] 	= $invoice_items;
		$this->data['invoice_num'] 		= $invoice_details->invoice_number;
		$this->data['invoice_date'] 	= $invoice_details->created;
		$this->data['invoice_due_date'] = $invoice_details->due_date;
		$this->data['invoice_status'] 	= $invoice_details->invoice_status;

		$invoice_total = $this->invoice_mdl->get_invoice_total_amt($invoice_id);
		$invoice_paid_amt = $this->invoice_mdl->get_invoice_paid_amt($invoice_id);
		$invoice_amt_due = $invoice_total - $invoice_paid_amt;
		
		
		$this->data['invoice_items'] = $invoice_items;
		$this->data['invoice_tax']   = format_amount ($invoice_tax, true);
		$this->data['invoice_subtotal']   = format_amount ($invoice_subtotal, true);
		$this->data['invoice_total'] = format_amount ($invoice_total, true);
		$this->data['invoice_paid_amt'] = format_amount ($invoice_paid_amt, true);
		$this->data['invoice_amt_due'] = format_amount ($invoice_amt_due, true);
		
		//get system settings information
		$this->data['settings']  = $this->settings_mdl->get(null, true);
		$this->data['pagecontent'] = 'frontend/view_invoice';
		$this->load->view('components/_layout', $this->data);
	}
	
	public function pay($invoice_id = null){
		//get the invoice details
		$this->db->where('invoice_id', $invoice_id);
		$invoice_details = $this->invoice_mdl->get(null, true);
		
		//Pre save the payment data
		$payment_details = array(
			'invoice_id'  		=> $invoice_id,
			'payment_amount'  	=> number_format($this->invoice_mdl->get_invoice_total_amt($invoice_id), 2),
			'payment_method'  	=> 'paypal',
			'payment_date'  	=> date('Y-m-d H:i:s'),
			'payment_note'      => 'invoice - ' . $invoice_details->invoice_number,
			'payment_status'	=> 'pending'
		);
		$this->load->model('payment_mdl');
		$payment_id = $this->payment_mdl->save($payment_details);
		
		//get the paypal details
		$this->load->model('paypal_mdl');
		$paypal_settings = $this->paypal_mdl->get(null, true);
		if($paypal_settings->active == 'YES' && $paypal_settings->paypal_account != ''){
		$this->load->library('paypal');
		$this->paypal->add_field('currency_code', strtoupper($paypal_settings->paypal_currency));
		$this->paypal->add_field('business', $paypal_settings->paypal_account);
		$this->paypal->add_field('return',  site_url('cinvoices/success/'.$invoice_id)); // return url
		$this->paypal->add_field('rm',  '2'); 
		$this->paypal->add_field('cancel_return', site_url('cinvoices/view/'.$invoice_id)); // cancel url
		$this->paypal->add_field('notify_url', $paypal_settings->paypal_ipn); // notify url
		$this->paypal->add_field('item_name', 'Invoice - '.$invoice_details->invoice_number);
		$this->paypal->add_field('amount', number_format($this->invoice_mdl->get_invoice_total_amt($invoice_id), 2));
				
		$this->paypal->add_field('custom', $payment_id);
		$this->paypal->submit_paypal_post(); // submit the fields to paypal
		}
		else {
			redirect('cinvoices/view/'.$invoice_id);
		}		
	}

/*---------------------------------------------------------------------------------------------------------
 | Function to display the pdf invoice
|----------------------------------------------------------------------------------------------------------*/
	public function invoice_pdf($invoice_id = null, $stream = true){
		$this->load->library("pdf");
		$this->load->model('settings_mdl');
		// Get the project details
		$this->db->select('*, clients.hourly_rate AS client_hourly_rate, projects.hourly_rate AS project_hourly_rate');
		$this->db->join('clients', 'clients.client_id = invoices.client_id', 'LEFT');
		$this->db->join('projects', 'projects.project_id = invoices.project_id', 'LEFT');
		$this->db->where('invoice_id', $invoice_id);
		$invoice_details = $this->invoice_mdl->get(null, true);
	
		//set the client information
		$client = new stdClass();
		$client->name 		= ucwords($invoice_details->firstname . ' ' . $invoice_details->lastname);
		$client->address 	= $invoice_details->address;
		$client->city 		= $invoice_details->city;
		$client->country 	= country_name($invoice_details->country);
		$client->company 	= $invoice_details->company;
		$client->client_id 	= $invoice_details->client_id;
		$client->invoice_id = $invoice_details->invoice_id;
		$this->data['client'] = $client;
	
		//Set project information
		$project = new stdClass();
		$project->project_name = $invoice_details->project_name;
		$project->project_id   = $invoice_details->project_id;
		$this->data['project'] = $project;
	
		//Get the time entries for that project
		$this->db->where('invoice_id = ', $invoice_id);
		$time_entries = $this->invoice_items_mdl->get();
		$invoice_items = array();
		$invoice_tax 		= 0;
		$invoice_subtotal 	= 0;
		foreach ($time_entries as $count => $entry) :
		$invoice_items[$count]['entry_id']	   = $entry->time_entry_id;
		$invoice_items[$count]['item_id']      = $entry->item_id;
		$invoice_items[$count]['description']  = $entry->description;
		$invoice_items[$count]['hours_worked'] = $entry->hours;
		$invoice_items[$count]['hourly_rate']  = $entry->rate;
		$invoice_items[$count]['item_amount']  = $entry->amount;
		$invoice_items[$count]['item_tax'] 	   = $entry->tax;
		$invoice_subtotal 					  += $invoice_items[$count]['item_amount'];
		$invoice_tax 						  += ($entry->tax * $invoice_items[$count]['item_amount']) / 100;
		endforeach;
	
		$this->data['invoice_items'] 	= $invoice_items;
		$this->data['invoice_num'] 		= $invoice_details->invoice_number;
		$this->data['invoice_date'] 	= $invoice_details->created;
		$this->data['invoice_due_date'] = $invoice_details->due_date;
		$this->data['invoice_status'] 	= $invoice_details->invoice_status;
	
		$invoice_total = $this->invoice_mdl->get_invoice_total_amt($invoice_id);
		$invoice_paid_amt = $this->invoice_mdl->get_invoice_paid_amt($invoice_id);
		$invoice_amt_due = $invoice_total - $invoice_paid_amt;
	
		$this->data['invoice_items'] = $invoice_items;
		$this->data['invoice_tax']   = format_amount ($invoice_tax, true);
		$this->data['invoice_subtotal']   = format_amount ($invoice_subtotal, true);
		$this->data['invoice_total'] = format_amount ($invoice_total, true);
		$this->data['invoice_paid_amt'] = format_amount ($invoice_paid_amt, true);
		$this->data['invoice_amt_due'] = format_amount ($invoice_amt_due, true);
	
		//get system settings information
		$this->data['settings']  = $this->settings_mdl->get(null, true);
		$pdf_content =  $this->load->view('pdf_templates/invoice_pdf', $this->data, true);
		$filename = 'invoices_'. $invoice_details->invoice_number;
		$this->pdf->create_pdf($filename, $pdf_content, $stream, false);
	}
	
	public function success($invoice_id = null){
		$this->session->set_flashdata('success', 'Payment completed successfully');
		redirect('cinvoices/view/'.$invoice_id);
	}
}