<?php
class Projects extends Backend_Controller {
	
	function __construct() 
	{
		parent::__construct();
		$this->data['_menu'] = 'projects';
		$this->load->model('project_mdl');
	}
	
	function index() 
	{
		//Get projects from the database
		$count = $this->db->count_all_results('projects');
		// Set pagination
		$perpage = $this->pagination_config['per_page'];
		if ($count > $perpage) {
			$this->pagination_config['base_url'] = site_url($this->uri->segment(1) .'/index/');
			$this->pagination_config['total_rows'] = $count;		
			$this->pagination_config['uri_segment'] = 3;
			$this->pagination->initialize($this->pagination_config);
			$this->data['pagination'] = $this->pagination->create_links();
			$offset = $this->uri->segment(3);
		}
		else{
			$this->data['pagination'] = '';
			$offset = 0;
		}
		$this->db->limit($perpage, $offset);
		$this->db->join('clients', 'clients.client_id = projects.client_id', 'LEFT');
		$this->data['projects'] = $this->project_mdl->get();
		$this->data['pagecontent'] = 'projects/list_projects';
		if(IS_AJAX)
			$this->load->view('projects/list_projects', $this->data);
		else
			$this->load->view('components/_layout', $this->data);
	}
	
	function project_form($project_id = null){
		//Fetch a user or set a new one
		if($project_id){
			$this->data['project'] = $this->project_mdl->get($project_id);
			count($this->data['project']) || $this->data['errors'][] = 'Project could not be found';
		}
		else {
			$this->data['project'] = $this->project_mdl->get_new();
		}
		$this->load->model('client_mdl');
		$this->data['clients'] = $this->client_mdl->clients_dropdown();
		$this->data['pagecontent'] = 'projects/project_form';
		$this->load->view('components/_layout', $this->data);
	}
	
	function save(){
		$id = ($this->input->post('project_id') != '') ? $this->input->post('project_id') : null;
		$rules = $this->project_mdl->project_rules;
		
		$this->form_validation->set_rules($rules);
		if($this->form_validation->run())
		{
			$post_fields = array('project_name', 'client_id', 'description', 'hourly_rate', 'estimated_hours', 'budget', 'project_status');
			$project_data = $this->project_mdl->array_from_post($post_fields);
			$this->project_mdl->save($project_data, $id);
			$response = array('success' => 1);
			$this->session->set_flashdata('success', 'Project details saved successfully');
		}
		else
		{
			$response = array(
					'success'           => 0,
					'validation_errors' => json_errors()
			);
		
		}
		echo json_encode($response);
	}
	public function delete($project_id = null){
		$this->project_mdl->delete_project($project_id);
		$this->session->set_flashdata('success', 'Project has been deleted !!');
		redirect('projects');
	}
}