<?php
class Reports extends Backend_Controller {
	
	function __construct() 
	{
		parent::__construct();
		$this->data['_menu'] = 'reports';
		$this->load->model('invoice_mdl');
		$this->load->model('invoice_items_mdl');
		$this->load->model('timer_mdl');
		$this->load->model('client_mdl');	
		$this->load->model('project_mdl');
		$this->load->model('payment_mdl');
		$this->load->library("Pdf");
	}
	
	public function index() 
	{
		/*---------------------------------------------------------------------------------------------------------
		 | Get the project details
		|----------------------------------------------------------------------------------------------------------*/
		$projects = new stdClass ();
		$this->db->where('project_status', 'closed');
		$projects_data = count($this->project_mdl->get());
		$projects->closed_projects = $projects_data;
		$projects->total_projects = count($this->project_mdl->get());
		$this->data['projects'] = $projects;
		
		/*---------------------------------------------------------------------------------------------------------
		 | Get hours worked
		|----------------------------------------------------------------------------------------------------------*/
		$this->data['total_hours_clocked'] = $this->timer_mdl->total_hours_clocked();
		
		/*---------------------------------------------------------------------------------------------------------
		 | Get all the invoices created
		|----------------------------------------------------------------------------------------------------------*/
		$this->data['total_invoices'] = count($this->invoice_mdl->get());
		
		/*---------------------------------------------------------------------------------------------------------
		 | Get all the payment received
		|----------------------------------------------------------------------------------------------------------*/
		$this->db->select_sum('payment_amount', 'total_earnings');
		$earnings = $this->payment_mdl->get(null, true);
		$this->data['total_received'] = format_amount($earnings->total_earnings, true);
		/*---------------------------------------------------------------------------------------------------------
		 | Get yearly earnings
		|----------------------------------------------------------------------------------------------------------*/
		$monthly_earnings = $this->payment_mdl->total_earnings();
		$earnings = array ();
		foreach ($monthly_earnings as $key=>$value) :
		foreach ($value as $month => $amount) :
		$date = date_parse($month);
		$earnings[] = array('month' => $month, 'earnings' => $amount);
		endforeach;
		endforeach;
		$this->data['yearly_earnings'] = json_encode($earnings);
		/*---------------------------------------------------------------------------------------------------------
		 | Get hours worked per month
		|----------------------------------------------------------------------------------------------------------*/
		$monthly_hours = $this->timer_mdl->monthly_hours();
		$hours_recorded = array ();
		foreach ($monthly_hours as $key=>$value) : 
			foreach ($value as $month => $hours) :
			$date = date_parse($month);
			$hours_recorded[] = array('month' => date('Y') . '-' . $date['month'], 'hours' => number_format($hours/3600, 3) );
			endforeach;
		endforeach;
		
		$this->data['monthly_hours'] = json_encode($hours_recorded);
		$this->data['reportmenu'] = 'general';
		$this->data['reportpage'] = 'reports/general';
		$this->data['pagecontent'] = 'reports/reports';
		$this->load->view('components/_layout', $this->data);
	}
	
	public function invoices(){
		//Get invoices from the database
		$this->db->join('projects', 'projects.project_id = invoices.project_id');
		$this->db->join('clients', 'clients.client_id = invoices.client_id');
		
		//Get all invoices from the database
		$count = $this->db->count_all_results('invoices');
		
		// Set pagination
		$perpage = $this->pagination_config['per_page'];
		if ($count > $perpage) {
			$this->pagination_config['base_url'] = site_url($this->uri->segment(1) .'/index/');
			$this->pagination_config['total_rows'] = $count;
			$this->pagination_config['uri_segment'] = 3;
			$this->pagination->initialize($this->pagination_config);
			$this->data['pagination'] = $this->pagination->create_links();
			$offset = $this->uri->segment(3);
		}
		else{
			$this->data['pagination'] = '';
			$offset = 0;
		}
		
		//Get invoices from the database
		$this->db->limit($perpage, $offset);
		$this->db->join('projects', 'projects.project_id = invoices.project_id');
		$this->db->join('clients', 'clients.client_id = invoices.client_id');
		
		$invoices = $this->invoice_mdl->get();
		$invoice_records = array ();
		foreach ($invoices as $count => $invoice) :
		$invoice_records[$count]['client_name']		=	$invoice->firstname . ' ' . $invoice->lastname;
		$invoice_records[$count]['project_name']	=	$invoice->project_name;
		$invoice_records[$count]['invoice_number']	=	$invoice->invoice_number;
		$invoice_records[$count]['invoice_status']	=	$invoice->invoice_status;
		$invoice_records[$count]['invoice_amount']	=	format_amount($this->invoice_mdl->get_invoice_total_amt($invoice->invoice_id), true);
		$invoice_records[$count]['paid_amount']		=	format_amount($this->invoice_mdl->get_invoice_paid_amt($invoice->invoice_id), true);
		$invoice_records[$count]['invoice_id']		=	$invoice->invoice_id;
		endforeach;
		
		$this->data['invoices'] = $invoice_records;

		$this->data['reportmenu'] = 'invoices';
		$this->data['reportpage'] = 'reports/invoices';
		$this->data['pagecontent'] = 'reports/reports';

		if(IS_AJAX)
			$this->load->view('billing/invoices', $this->data);
		else
			$this->load->view('components/_layout', $this->data);

	}
	public function projects(){
		
		//Get projects from the database
		$count = $this->db->count_all_results('projects');
		// Set pagination
		$perpage = $this->pagination_config['per_page'];
		if ($count > $perpage) {
			$this->pagination_config['base_url'] = site_url($this->uri->segment(1) .'/index/');
			$this->pagination_config['total_rows'] = $count;
			$this->pagination_config['uri_segment'] = 3;
			$this->pagination->initialize($this->pagination_config);
			$this->data['pagination'] = $this->pagination->create_links();
			$offset = $this->uri->segment(3);
		}
		else{
			$this->data['pagination'] = '';
			$offset = 0;
		}
		$this->db->limit($perpage, $offset);
		$this->db->join('clients', 'clients.client_id = projects.client_id', 'LEFT');
		$this->data['projects'] = $this->project_mdl->get();
		
		$this->data['reportmenu'] = 'projects';
		$this->data['reportpage'] = 'reports/projects';
		$this->data['pagecontent'] = 'reports/reports';
		
		if(IS_AJAX)
			$this->load->view('projects/list_projects', $this->data);
		else
			$this->load->view('components/_layout', $this->data);
		
		
	}
	public function payments(){
		$this->db->select ('payments.*, clients.firstname, clients.lastname, clients.company'); 
		$this->db->join('invoices', 'invoices.invoice_id = payments.invoice_id');
		$this->db->join('clients', 'clients.client_id = invoices.client_id');
		$this->data['payments'] = $this->payment_mdl->get();
		$this->data['reportmenu'] = 'payments';
		$this->data['reportpage'] = 'reports/payments';
		$this->data['pagecontent'] = 'reports/reports';
		$this->load->view('components/_layout', $this->data);
	}
	public function clients(){
		//Get clients from the database
		$this->data['clients'] = $this->client_mdl->get();
		$this->data['reportmenu'] = 'clients';
		$this->data['reportpage'] = 'reports/clients';
		$this->data['pagecontent'] = 'reports/reports';
		$this->load->view('components/_layout', $this->data);
	}
/*---------------------------------------------------------------------------------------------------------
| Generate client list pdf 
|----------------------------------------------------------------------------------------------------------*/
	public function clients_pdf(){
		//Get clients from the database
		$this->data['clients'] = $this->client_mdl->get();
		$pdf_content = $this->load->view('reports/pdf_templates/clients_pdf', $this->data, true);
		$filename = 'clients_list_' . format_date(date('d-m-Y'));
		$this->pdf->create_pdf($filename, $pdf_content);
	}
/*---------------------------------------------------------------------------------------------------------
| Generate client list pdf
|----------------------------------------------------------------------------------------------------------*/
	public function projects_pdf(){
		//Get projects from the database
		$this->db->join('clients', 'clients.client_id = projects.client_id', 'LEFT');
		$this->data['projects'] = $this->project_mdl->get();
		$pdf_content = $this->load->view('reports/pdf_templates/projects_pdf', $this->data, true);
		$filename = 'projects_list_' . format_date(date('d-m-Y'));
		$this->pdf->create_pdf($filename, $pdf_content);
	}
/*---------------------------------------------------------------------------------------------------------
 | Generate payments pdf
|----------------------------------------------------------------------------------------------------------*/
	public function payments_pdf(){
		//Get payments from the database
		$this->db->select ('payments.*, clients.firstname, clients.lastname, clients.company'); 
		$this->db->join('invoices', 'invoices.invoice_id = payments.invoice_id');
		$this->db->join('clients', 'clients.client_id = invoices.client_id');
		$this->data['payments'] = $this->payment_mdl->get();
		$pdf_content = $this->load->view('reports/pdf_templates/payments_pdf', $this->data, true);
		$filename = 'payments_list_' . format_date(date('d-m-Y'));
		$this->pdf->create_pdf($filename, $pdf_content);
	}
/*---------------------------------------------------------------------------------------------------------
| Generate invoice list pdf
|----------------------------------------------------------------------------------------------------------*/
	public function invoices_pdf(){
		//Get the invoices from the database
		$this->db->join('projects', 'projects.project_id = invoices.project_id');
		$this->db->join('clients', 'clients.client_id = invoices.client_id');
		
		$invoices = $this->invoice_mdl->get();
		$invoice_records = array ();
		foreach ($invoices as $count => $invoice) :
		$invoice_records[$count]['client_name']		=	$invoice->firstname . ' ' . $invoice->lastname;
		$invoice_records[$count]['project_name']	=	$invoice->project_name;
		$invoice_records[$count]['invoice_number']	=	$invoice->invoice_number;
		$invoice_records[$count]['invoice_status']	=	$invoice->invoice_status;
		$invoice_records[$count]['invoice_amount']	=	format_amount($this->invoice_mdl->get_invoice_total_amt($invoice->invoice_id), true);
		$invoice_records[$count]['paid_amount']		=	format_amount($this->invoice_mdl->get_invoice_paid_amt($invoice->invoice_id), true);
				
		$invoice_records[$count]['invoice_id']		=	$invoice->invoice_id;
		endforeach;
		
		$this->data['invoices'] = $invoice_records;
		$pdf_content = $this->load->view('reports/pdf_templates/invoices_pdf', $this->data, true);
		$filename = 'invoices_list_' . format_date(date('d-m-Y'));
		$this->pdf->create_pdf($filename, $pdf_content);
	}
}