<?php
class Support extends Backend_Controller {
	
	function __construct() 
	{
		parent::__construct();
		$this->data['_menu'] = 'support';
		$this->load->model('client_mdl');
		$this->load->model('user_mdl');
		$this->load->model('tickets_mdl');
	}
/*-----------------------------------------------------------------------------------------------------------------------
 function to list all the tickets from the database
-------------------------------------------------------------------------------------------------------------------------*/

	function index($filter = 'all') 
	{
		//Get clients from the database
		$count = count($this->tickets_mdl->get_tickets($filter));
		// Set pagination
		$perpage = $this->pagination_config['per_page'];
		if ($count > $perpage) {
			$this->pagination_config['base_url'] = site_url($this->uri->segment(1) .'/index/'.$filter.'/');
			$this->pagination_config['total_rows'] = $count;
			$this->pagination_config['uri_segment'] = 4;
			$this->pagination->initialize($this->pagination_config);
			$this->data['pagination'] = $this->pagination->create_links();
			$offset = $this->uri->segment(4);
		}
		else{
			$this->data['pagination'] = '';
			$offset = 0;
		}
		$this->db->limit($perpage, $offset);

		$this->data['tickets'] = $this->tickets_mdl->get_tickets($filter);
		$this->data['clients'] = $this->client_mdl->clients_dropdown();
		$this->data['agents']  = $this->user_mdl->agents_dropdown();
		$this->data['filter_by'] = $filter;
		
		$this->data['pagecontent'] = 'support/tickets';
		
		if(IS_AJAX)
			$this->load->view('support/tickets', $this->data);
		else
			$this->load->view('components/_layout', $this->data);
	}
	
/*-----------------------------------------------------------------------------------------------------------------------
 function to save a new ticket
-------------------------------------------------------------------------------------------------------------------------*/

	function save_ticket()
	{
		$id = ($this->input->post('ticket_id') != '') ? $this->input->post('ticket_id') : null;
		!$id ? $rules = $this->tickets_mdl->ticket_rules : $rules = $this->tickets_mdl->update_ticket_rules;

		$this->form_validation->set_rules($rules);
		if($this->form_validation->run())
		{
			!$id ? $post_fields = array('client_id', 'assigned_to', 'subject', 'ticket_body') : $post_fields = array('client_id', 'assigned_to', 'ticket_status');
			$ticket_data = $this->tickets_mdl->array_from_post($post_fields);
			if(!$id) $ticket_data['ticket_status'] = 'open' ;
			
			if(isset($_FILES['ticket_attachment']['name']) && $_FILES['ticket_attachment']['name'] != '')
			{
				$upload_response = $this->upload_attachment('ticket_attachment');
				
				if($upload_response['success'])
				{
					$upload_data = $upload_response['upload_data'];
					$ticket_data['attachment'] = $upload_data['file_name'];

					$ticket_id = $this->tickets_mdl->save($ticket_data, $id);
					$ticket_number = array('ticket_number' => str_pad($ticket_id, 4, "0", STR_PAD_LEFT));
					$ticket_id = $this->tickets_mdl->save($ticket_number, $ticket_id);					
					$response = array('success' => 1);
					$this->session->set_flashdata('success', 'Ticket saved successfully');
				}
				else 
				{
						$fail_error = $upload_response['msg'];
						$response = array(
								'success'           => 0,
								'invalid_upload'	=> true,
								'upload_error'		=> $fail_error,
								'validation_errors' => json_errors()
						);
				}
			}
			else 
			{
				$ticket_id = $this->tickets_mdl->save($ticket_data, $id);
				if(!$id){
				$ticket_number = array('ticket_number' => str_pad($ticket_id, 4, "0", STR_PAD_LEFT));
				$ticket_id = $this->tickets_mdl->save($ticket_number, $ticket_id);
				}

				$response = array('success' => 1);
				$this->session->set_flashdata('success', 'Ticket saved successfully');
			}
			if(!$id){
				//Create notification details array
				if($this->session->userdata('user_group') == 3){
					$this->notifications->new_ticket_admin($ticket_id);
				}
				else{
					$assigned_to = $this->input->post('assigned_to');
					if($assigned_to != ''){
						//Notify the assigned agent
						$this->notifications->new_ticket_admin ($ticket_id, true);
					}
					$this->notifications->new_ticket_client($ticket_id);
				}
			}
		}
		else
		{
			$response = array(
					'success'           => 0,
					'validation_errors' => json_errors()
			);
		
		}

		echo json_encode($response);
	}

/*-----------------------------------------------------------------------------------------------------------------------
	function to upload the ticket attachment
-------------------------------------------------------------------------------------------------------------------------*/
	
	function upload_attachment($fieldname) {
		//set the path where the files uploaded will be copied. NOTE if using linux, set the folder to permission 777
		$config['upload_path'] = ATTACHMENTS;
		// set the filter image types
		$config['allowed_types'] = 'gif|jpg|png|pdf|doc|docx|xls|ppt|pptx';
		$config['width'] = 150;
		$this->load->helper('string');
		$config['file_name']	 = random_string('alnum', 32);
		//load the upload library
		$this->load->library('upload', $config);
		$this->upload->initialize($config);
	
		//if not successful, set the error message
		if (!$this->upload->do_upload($fieldname)) {
			$data = array('success' => false, 'msg' => $this->upload->display_errors());
		}
		else
		{ //else, set the success message
			$upload_details = $this->upload->data();
			$data = array('success' => true, 'upload_data' => $upload_details, 'msg' => "Upload success!");
		}
		return $data;
	}
/*-----------------------------------------------------------------------------------------------------------------------
 function to view ticket
-------------------------------------------------------------------------------------------------------------------------*/
	function view($ticket_id = null){
		$this->data['clients'] = $this->client_mdl->clients_dropdown();
		$this->data['agents']  = $this->user_mdl->agents_dropdown();
		$this->data['ticket'] = $this->tickets_mdl->view_ticket($ticket_id);
		$this->data['pagecontent'] = 'support/view_ticket';
		$this->load->view('components/_layout', $this->data);
	}
/*-----------------------------------------------------------------------------------------------------------------------
 function to save ticket messages
-------------------------------------------------------------------------------------------------------------------------*/

	function save_ticket_message(){
		$rules = $this->tickets_mdl->ticket_message_rules;
		$this->tickets_mdl->_table_name = 'ticket_messages';
		$this->tickets_mdl->_primary_key = 'message_id';
		$this->form_validation->set_rules($rules);
		if($this->form_validation->run())
		{
			
			$post_fields = array('ticket_id', 'sender', 'sender_id', 'message_content');
			$message_data = $this->tickets_mdl->array_from_post($post_fields);
				
			if($_FILES['ticket_attachment']['name'] != '')
			{
				$upload_response = $this->upload_attachment('ticket_attachment');
		
				if($upload_response['success'])
				{
					$upload_data = $upload_response['upload_data'];
					$message_data['attachment'] = $upload_data['file_name'];
		
					$this->tickets_mdl->save($message_data);
					$response = array('success' => 1);
					$this->session->set_flashdata('success', 'Reply sent successfully');

					$this->tickets_mdl->_table_name = 'tickets';
					$this->tickets_mdl->_primary_key = 'ticket_id';
					if($this->session->userdata('user_group') == 3){
						$this->notifications->new_ticket_message_admin($this->input->post('ticket_id'), $message_data['message_content']);
					}
					else {
						$this->notifications->new_ticket_message_client($this->input->post('ticket_id'), $message_data['message_content']);
					}
				}
				else
				{
					$fail_error = $upload_response['msg'];
					$response = array(
							'success'           => 0,
							'invalid_upload'	=> true,
							'upload_error'		=> $fail_error,
							'validation_errors' => json_errors()
					);
				}
			}
			else
			{
				$ticket_message_id = $this->tickets_mdl->save($message_data);
				$response = array('success' => 1);
				$this->session->set_flashdata('success', 'Reply sent successfully');

				$this->tickets_mdl->_table_name = 'tickets';
				$this->tickets_mdl->_primary_key = 'ticket_id';
				if($this->session->userdata('user_group') == 3){
					$this->notifications->new_ticket_message_admin($this->input->post('ticket_id'), $message_data['message_content']);
				}
				else {
					$this->notifications->new_ticket_message_client($this->input->post('ticket_id'), $message_data['message_content']);
				}
			}
			
		}
		else
		{
			$response = array(
					'success'           => 0,
					'validation_errors' => json_errors()
			);
		
		}
		echo json_encode($response);
	}
}