<?php
class Users extends Backend_Controller {
	
	function __construct() 
	{
		parent::__construct();
		$this->data['_menu'] = 'users';
	}
	
	function index() 
	{
		//Get users from the database
		$this->db->where('id != ', $this->session->userdata('id'));
		$count = $this->db->count_all_results('users');
		// Set pagination
		$perpage = $this->pagination_config['per_page'];
		if ($count > $perpage) {
			$this->pagination_config['base_url'] = site_url($this->uri->segment(1) .'/index/');
			$this->pagination_config['total_rows'] = $count;
			$this->pagination_config['uri_segment'] = 3;
			$this->pagination->initialize($this->pagination_config);
			$this->data['pagination'] = $this->pagination->create_links();
			$offset = $this->uri->segment(3);
		}
		else{
			$this->data['pagination'] = '';
			$offset = 0;
		}
		$this->db->limit($perpage, $offset);
		$this->db->where('id != ', $this->session->userdata('id'));
		$this->db->where('user_group != ', '3');
		$this->data['users'] = $this->user_mdl->get();
		$this->data['pagecontent'] = 'users/list_users';
		
		// Load appropriate view
		if(IS_AJAX)
			$this->load->view('users/list_users', $this->data);
		else
			$this->load->view('components/_layout', $this->data);
	}
	
	public function user_form($id = null){
		//Fetch a user or set a new one
		if($id){
			$this->data['user'] = $this->user_mdl->get($id);
			count($this->data['user']) || $this->data['errors'][] = 'User could not be found';
		}
		else {
			$this->data['user'] = $this->user_mdl->get_new();
		}
		
		$this->data['pagecontent'] = 'users/user_form';
		$this->load->view('components/_layout', $this->data);
	}
	
	public function save(){
		
		$id = ($this->input->post('user_id') != '') ? $this->input->post('user_id') : null;
		$rules = $this->user_mdl->register_rules;
		if(!$id) $rules['password']['rules'] .= '|required';
		
		$this->form_validation->set_rules($rules);
		if($this->form_validation->run())
		{
			$post_fields = array('firstname', 'lastname', 'email', 'password', 'telephone', 'user_group');
			$account_data = $this->user_mdl->array_from_post($post_fields);
			if($account_data['password'] != ''){ 
				$account_data['password'] = $this->user_mdl->hash($account_data['password']);
			}
			else {
				unset($account_data['password']);
			}
			
			$user_notification = new stdClass();
			$user_notification->username = $this->input->post('firstname');
			$user_notification->password = $this->input->post('password');
			$user_notification->email = $this->input->post('email');
			
			if($_FILES['profile_pic']['name'] != '')
			{
				$upload_response = $this->profile_picture('profile_pic');
				
				if($upload_response['success'])
				{
					$upload_data = $upload_response['upload_data'];
					$account_data['profile_pic'] = $upload_data['file_name'];
					
					$user_details = $this->user_mdl->get($id, true);
					$user_id = $this->user_mdl->save($account_data, $id);
					if($user_id)
					{
						if(!empty($user_details->profile_pic))
						{
							if(is_file(PROFILEPICSFOLDER.$user_details->profile_pic))
								unlink(PROFILEPICSFOLDER.$user_details->profile_pic);
						}
						$response = array('success' => 1);
						$this->session->set_flashdata('success', 'User details saved successfully');
					}
					//Send user login details via email
					if($id == null)
					$this->notifications->login_details($user_notification);
				}
				else 
				{
						$fail_error = $upload_response['msg'];
						$response = array(
								'success'           => 0,
								'invalid_upload'	=> true,
								'upload_error'		=> $fail_error,
								'validation_errors' => json_errors()
						);
				}
			}
			else 
			{
				$user_id = $this->user_mdl->save($account_data, $id);
				//Send user login details via email
				if($id == null)
				$this->notifications->login_details($user_notification);
				$response = array('success' => 1);
				$this->session->set_flashdata('success', 'User details saved successfully');
			}
		}
		else
		{
			$response = array(
					'success'           => 0,
					'validation_errors' => json_errors()
			);
		
		}
		echo json_encode($response);
	}
	
	//function to save personal profile
	public function save_profile(){
	
		$id = ($this->input->post('user_id') != '') ? $this->input->post('user_id') : null;
		$rules = $this->user_mdl->profile_rules;
		if(!$id) $rules['password']['rules'] .= '|required';
	
		$this->form_validation->set_rules($rules);
		if($this->form_validation->run())
		{
			$post_fields = array('firstname', 'lastname', 'email', 'password', 'telephone');
			$account_data = $this->user_mdl->array_from_post($post_fields);
			if($account_data['password'] != ''){
				$account_data['password'] = $this->user_mdl->hash($account_data['password']);
			}
			else {
				unset($account_data['password']);
			}
				
			if($_FILES['profile_pic']['name'] != '')
			{
				$upload_response = $this->profile_picture('profile_pic');
	
				if($upload_response['success'])
				{
					$upload_data = $upload_response['upload_data'];
					$account_data['profile_pic'] = $upload_data['file_name'];
						
					$user_details = $this->user_mdl->get($id, true);
					$user_id = $this->user_mdl->save($account_data, $id);
					if($user_id)
					{
						if(!empty($user_details->profile_pic))
						{
							if(is_file(PROFILEPICSFOLDER.$user_details->profile_pic))
								unlink(PROFILEPICSFOLDER.$user_details->profile_pic);
						}
						$response = array('success' => 1);
						$this->session->set_flashdata('success', 'User details saved successfully');
						$this->user_mdl->update_session_data();
					}
				}
				else
				{
					$fail_error = $upload_response['msg'];
					$response = array(
							'success'           => 0,
							'invalid_upload'	=> true,
							'upload_error'		=> $fail_error,
							'validation_errors' => json_errors()
					);
				}
			}
			else
			{
				$user_id = $this->user_mdl->save($account_data, $id);
				$response = array('success' => 1);
				$this->session->set_flashdata('success', 'User details saved successfully');
				$this->user_mdl->update_session_data();
			}
		}
		else
		{
			$response = array(
					'success'           => 0,
					'validation_errors' => json_errors()
			);
		}
		echo json_encode($response);
	}
	
	
	// function to upload user profile picture	
	
	function profile_picture($fieldname) {
		//set the path where the files uploaded will be copied. NOTE if using linux, set the folder to permission 777
		$config['upload_path'] = PROFILEPICSFOLDER;
		// set the filter image types
		$config['allowed_types'] = 'gif|jpg|png';
		$config['width'] = 150;
		$this->load->helper('string');
		$config['file_name']	 = random_string('alnum', 32);
		//load the upload library
		$this->load->library('upload', $config);
		$this->upload->initialize($config);
	
		//if not successful, set the error message
		if (!$this->upload->do_upload($fieldname)) {
			$data = array('success' => false, 'msg' => $this->upload->display_errors());
		} 
		else 
		{ //else, set the success message
			$upload_details = $this->upload->data();
			$logo_location = $config['upload_path'].$upload_details['file_name'];
			$this->resize_image($logo_location);
			$data = array('success' => true, 'upload_data' => $upload_details, 'msg' => "Upload success!");
		}
		return $data; 
	}
	
	function resize_image($img_location){
		$this->load->library('image_lib');
		$configThumb = array();
		$configThumb['image_library']   = 'gd2';
		$configThumb['maintain_ratio']  = TRUE;
		$configThumb['source_image']	= $img_location;
		$configThumb['width']           = 150;
		$configThumb['height']          = 150;
		$this->image_lib->initialize($configThumb);
		$this->image_lib->resize();
	}
	
	public function _unique_email($str){
	
		//DO NOT validate if email exists
		//UNLESS it the email for the current user
	
		$id = $this->uri->segment(3);
		$this->db->where('email', $this->input->post('email'));
		!$id || $this->db->where('id != ', $id);
		$user = $this->user_mdl->get();
	
		if(count($user)){
			$this->form_validation->set_message('_unique_email', '%s should be unique');
			return FALSE;
		}
		return TRUE;
	}
/*---------------------------------------------------------------------------------------------------------
 | Function to display user profile
|----------------------------------------------------------------------------------------------------------*/
	
	public function profile(){
		$id = $this->session->userdata('id');
		$this->data['user'] = $this->user_mdl->get($id);
		count($this->data['user']) || $this->data['errors'][] = 'User could not be found';
		$this->data['_menu'] = 'profile';
		$this->data['pagecontent'] = 'users/profile';
		$this->load->view('components/_layout', $this->data);
	}
/*---------------------------------------------------------------------------------------------------------
 | Function to delete users
|----------------------------------------------------------------------------------------------------------*/
	public function delete($user_id = null){
		$user = $this->user_mdl->get($user_id, true);
		$profile_picture = $user->profile_pic;
		delete_profile_pic($profile_picture);
		$this->user_mdl->delete($user_id);
		$this->session->set_flashdata('success', 'User has been deleted !!');
		redirect('users');
	}

}