--
-- Table structure for table `ci_sessions`
--

CREATE TABLE IF NOT EXISTS `ci_sessions` (
  `session_id` varchar(40) NOT NULL DEFAULT '0',
  `ip_address` varchar(16) NOT NULL DEFAULT '0',
  `user_agent` varchar(120) DEFAULT NULL,
  `last_activity` int(10) unsigned NOT NULL DEFAULT '0',
  `user_data` text NOT NULL,
  PRIMARY KEY (`session_id`),
  KEY `last_activity` (`last_activity`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

-- --------------------------------------------------------

--
-- Table structure for table `clients`
--

CREATE TABLE IF NOT EXISTS `clients` (
  `client_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `firstname` varchar(100) NOT NULL,
  `lastname` varchar(100) NOT NULL,
  `email` varchar(100) NOT NULL,
  `uacc_fk` int(11) NOT NULL,
  `hourly_rate` double NOT NULL,
  `address` varchar(100) NOT NULL,
  `company` varchar(100) NOT NULL,
  `city` varchar(100) NOT NULL,
  `country` varchar(100) NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`client_id`)
) ;

-- --------------------------------------------------------

--
-- Table structure for table `config`
--

CREATE TABLE IF NOT EXISTS `config` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `company_name` varchar(100) NOT NULL,
  `address` varchar(100) NOT NULL,
  `city` varchar(100) NOT NULL,
  `email` varchar(250) NOT NULL,
  `phone` varchar(100) NOT NULL,
  `website` varchar(100) NOT NULL,
  `timezone` varchar(100) NOT NULL,
  `company_logo` varchar(100) NOT NULL,
  `invoice_logo` varchar(100) NOT NULL,
  `tax` double NOT NULL,
  `date_format` varchar(100) NOT NULL,
  `currency` varchar(20) NOT NULL,
  `time_format` varchar(20) NOT NULL,
  `decimal_places` int(11) NOT NULL,
  `decimal_separator` char(1) NOT NULL,
  `thousand_separator` char(1) NOT NULL,
  `invoice_due_days` int(11) NOT NULL DEFAULT '14',
  PRIMARY KEY (`id`)
)  ;

-- --------------------------------------------------------

--
-- Table structure for table `invoices`
--

CREATE TABLE IF NOT EXISTS `invoices` (
  `invoice_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `invoice_number` varchar(255) NOT NULL,
  `client_id` int(11) NOT NULL,
  `project_id` int(11) NOT NULL,
  `invoice_status` enum('DRAFT','UNPAID','PAID') NOT NULL DEFAULT 'DRAFT',
  `due_date` datetime NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`invoice_id`)
) ;

-- --------------------------------------------------------

--
-- Table structure for table `invoice_items`
--

CREATE TABLE IF NOT EXISTS `invoice_items` (
  `item_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `invoice_id` varchar(255) NOT NULL,
  `time_entry_id` int(11) NOT NULL,
  `description` text NOT NULL,
  `hours` double NOT NULL,
  `rate` double NOT NULL,
  `tax` double NOT NULL,
  `amount` double NOT NULL,
  PRIMARY KEY (`item_id`)
);

-- --------------------------------------------------------

--
-- Table structure for table `migrations`
--

CREATE TABLE IF NOT EXISTS `migrations` (
  `version` int(3) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

--
-- Dumping data for table `migrations`
--

INSERT INTO `migrations` (`version`) VALUES
(13);

-- --------------------------------------------------------

--
-- Table structure for table `notifications`
--

CREATE TABLE IF NOT EXISTS `notifications` (
  `notification_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `to_user` int(11) NOT NULL,
  `subject` varchar(255) NOT NULL,
  `notification_content` text NOT NULL,
  `notification_link` varchar(255) NOT NULL,
  `seen` enum('1','0') NOT NULL DEFAULT '0',
  `timestamp` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
  PRIMARY KEY (`notification_id`)
);

-- --------------------------------------------------------

--
-- Table structure for table `payments`
--

CREATE TABLE IF NOT EXISTS `payments` (
  `payment_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `invoice_id` int(11) NOT NULL,
  `payment_amount` double NOT NULL,
  `payment_method` varchar(255) NOT NULL,
  `payment_date` date NOT NULL,
  `payment_note` text NOT NULL,
  `payment_status` varchar(255) NOT NULL,
  PRIMARY KEY (`payment_id`)
);

-- --------------------------------------------------------

--
-- Table structure for table `paypal_settings`
--

CREATE TABLE IF NOT EXISTS `paypal_settings` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `paypal_account` varchar(100) NOT NULL,
  `paypal_currency` varchar(100) NOT NULL,
  `paypal_ipn` varchar(100) NOT NULL,
  `active` enum('YES','NO') NOT NULL DEFAULT 'YES',
  PRIMARY KEY (`id`)
);

-- --------------------------------------------------------

--
-- Table structure for table `projects`
--

CREATE TABLE IF NOT EXISTS `projects` (
  `project_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `project_name` varchar(100) NOT NULL,
  `client_id` int(11) NOT NULL,
  `project_status` enum('open','closed') NOT NULL,
  `hourly_rate` double NOT NULL,
  `estimated_hours` double NOT NULL,
  `budget` double NOT NULL,
  `description` text NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`project_id`)
);

-- --------------------------------------------------------

--
-- Table structure for table `templates`
--

CREATE TABLE IF NOT EXISTS `templates` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `template_name` varchar(100) NOT NULL,
  `subject` varchar(100) NOT NULL,
  `body` text NOT NULL,
  PRIMARY KEY (`id`)
);

--
-- Dumping data for table `templates`
--

INSERT INTO `templates` (`id`, `template_name`, `subject`, `body`) VALUES
(1, 'ticket', 'Ticket Notification', '<div>{company_logo}</div>\r\n<div>&nbsp;\r\n<table class="body-wrap" bgcolor="#D9EDF7" width="80%" cellpadding="20px">\r\n <tbody><tr>\r\n  <td></td>\r\n  <td class="container">\r\n   <div class="content"><h4>Ticket Notification</h4><table>\r\n    <tbody><tr>\r\n     <td>\r\n       <p><span>Hi {username},</span></p>\r\n       <table><tbody><tr><td class="padding"><p> <strong>Ticket Number:</strong> {ticket_number} <span><strong>Ticket Date:</strong> {ticket_date}</span></p><p><span>{ticket_message}</span></p><p><span><br></span></p></td></tr></tbody></table>\r\n      <p>Kind Regards,&nbsp;</p><p>{company}</p><p><br></p>\r\n     </td>\r\n    </tr>\r\n   </tbody></table>\r\n   </div>\r\n  </td>\r\n  <td></td>\r\n </tr>\r\n</tbody></table>\r\n</div>'),
(2, 'invoice', 'You''ve received an invoice', '<div>{invoice_logo}</div>\r\n<div>&nbsp;\r\n<table class="body-wrap" bgcolor="#D9EDF7" width="80%" cellpadding="20px">\r\n <tbody><tr>\r\n  <td></td>\r\n  <td class="container">\r\n   <div class="content"><h4>Invoice Notification</h4><table>\r\n    <tbody><tr>\r\n     <td>\r\n       <p><span>Hi {username},</span></p>\r\n       Please find your invoice attached.<br>\r\n      <p><br></p><p>Kind Regards,&nbsp;</p><p>{company}</p><p><br></p>\r\n     </td>\r\n    </tr>\r\n   </tbody></table><br></div>\r\n  </td>\r\n  <td></td>\r\n </tr>\r\n</tbody></table>\r\n</div>'),
(3, 'password_reset', 'Password Reset notification', '<div>{company_logo}</div>\n<div>&nbsp;\n<table class="body-wrap" bgcolor="#D9EDF7" width="80%" cellpadding="20px">\n <tbody><tr>\n  <td></td>\n  <td class="container">\n   <div class="content"><h4>Password Reset Notification</h4><table>\n    <tbody><tr>\n     <td>\n       <p><span>Hi {username},</span></p>\n       Your password has been reset to : <b>{password}</b>.<br>\n       {login_link}\n      <p><br></p><p>Kind Regards,&nbsp;</p><p>{company}</p><p><br></p>\n     </td>\n    </tr>\n   </tbody></table><br></div>\n  </td>\n  <td></td>\n </tr>\n</tbody></table>\n</div>'),
(4, 'notification', 'Notification', '<div>{company_logo}</div>\n<div>&nbsp;\n<table class="body-wrap" bgcolor="#D9EDF7" width="80%" cellpadding="20px">\n <tbody><tr>\n  <td></td>\n  <td class="container">\n   <div class="content"><h4>Notification</h4><table>\n    <tbody><tr>\n     <td>\n       <p><span>Hi {username},</span></p>\n       {message}.<br>\n      <p><br></p><p>Kind Regards,&nbsp;</p><p>{company}</p><p><br></p>\n     </td>\n    </tr>\n   </tbody></table><br></div>\n  </td>\n  <td></td>\n </tr>\n</tbody></table>\n</div>'),
(5, 'login_details', 'Login Details', '<div>{company_logo}</div>\n<div>&nbsp;\n<table class="body-wrap" bgcolor="#D9EDF7" width="80%" cellpadding="20px">\n <tbody><tr>\n  <td></td>\n  <td class="container">\n   <div class="content"><h4>Login Details</h4><table>\n    <tbody><tr>\n     <td>\n       <p><span>Hi {username},</span></p>\n       Your login details are as follows, use the link below to login .<br>\n      <p>{login_link}<br></p>\n      <p>Username : {username}<br></p>\n      <p>Password : {password}<br></p><p><br></p>\n      <p>Kind Regards,&nbsp;</p><p>{company}</p><p><br></p>\n     </td>\n    </tr>\n   </tbody></table><br></div>\n  </td>\n  <td></td>\n </tr>\n</tbody></table>\n</div>');

-- --------------------------------------------------------

--
-- Table structure for table `tickets`
--

CREATE TABLE IF NOT EXISTS `tickets` (
  `ticket_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `ticket_number` varchar(255) NOT NULL,
  `client_id` int(11) NOT NULL,
  `subject` varchar(255) NOT NULL,
  `ticket_body` text NOT NULL,
  `attachment` varchar(255) NOT NULL,
  `assigned_to` int(11) NOT NULL,
  `ticket_status` enum('open','closed') NOT NULL DEFAULT 'open',
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`ticket_id`)
);

-- --------------------------------------------------------

--
-- Table structure for table `ticket_messages`
--

CREATE TABLE IF NOT EXISTS `ticket_messages` (
  `message_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `ticket_id` int(11) NOT NULL,
  `message_content` text NOT NULL,
  `attachment` varchar(255) NOT NULL,
  `sender` enum('client','agent') NOT NULL DEFAULT 'client',
  `sender_id` int(11) NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`message_id`)
);

-- --------------------------------------------------------

--
-- Table structure for table `timer_entries`
--

CREATE TABLE IF NOT EXISTS `timer_entries` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `user_id` int(11) NOT NULL,
  `project_id` int(11) NOT NULL,
  `task` varchar(255) NOT NULL,
  `start_time` int(11) NOT NULL DEFAULT '0',
  `stop_time` int(11) NOT NULL DEFAULT '0',
  `active` tinyint(1) NOT NULL DEFAULT '1',
  `is_invoiced` enum('0','1') NOT NULL DEFAULT '0',
  PRIMARY KEY (`id`)
);

-- --------------------------------------------------------

--
-- Table structure for table `users`
--

CREATE TABLE IF NOT EXISTS `users` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `firstname` varchar(100) NOT NULL,
  `lastname` varchar(100) NOT NULL,
  `email` varchar(100) NOT NULL,
  `password` varchar(255) NOT NULL,
  `telephone` varchar(255) NOT NULL,
  `user_group` enum('1','2','3') NOT NULL DEFAULT '2',
  `profile_pic` varchar(255) NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`id`)
);
